#!/usr/bin/perl
##
# journal.pl
# Airline Operations Journal
# Copyright 2004 D. Scott Barninger <barninger@fairfieldcomputers.com>
# Licensed under the GPL V2
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# version 1.0 18 Apr 2004
# version 1.1 28 Aug 2004
# - added pretty date formatting

#-------------------------------------------------------------------#
# Program Configuration
#
my $prog_name = "/cgi-bin/journal.pl";
my $spell_prog_name = "/cgi-bin/spell.pl";
my $manage_email_prog = "/cgi-bin/update_email.pl";
# hostname (or IP address) of smtp server
my $mailserver = "";
my $mailuser = "";
my $mailpass = "";
my $return_address = "username\@mydomain.com";
# MySQL database name, host, user & password
my $database = "database";
my $host = "localhost";
my $user = "dbuser";
my $password = "dbpassword";
# path to graphics from document root
my $logo_graphic = "/logo.gif";
my $spacer_graphic = "/clearpixel.gif";
# name and address information
my $airline_name = "My Airlines";
my $airline_address_1 = "Some International Airport";
my $airline_address_2 = "Anytown PA 20001";
# the time the official schedule day starts and ends
# note - changing these times requires matching changes in the
# DisplayRecord() and Mailit() subroutines
my $day_start = "04:00:00";
my $day_stop = "03:59:59";
my $copyright = "Copyright &copy; 2004 D. Scott Barninger";

#-------------------------------------------------------------------#

use DBI();
use Date::EzDate;
use Mail::SendEasy;

my %input = &getcgivars;
my $action = $input{'action'};
my $department = $input{'department'};
my $department_name;

&Main();
exit;

#-------------------------------------------------------------------#
# Function Main
# Description: check requested action and call appropriate subroutine
#
sub Main {
	# set default action & department
	if (!$action) {$action = "display"};
	if (!$department) {$department = "Ops"};
	if ($department eq "Ops") {$department_name = "Operations"};
	if ($department eq "Maint") {$department_name = "Maintenance"};

	if ($department ne "Ops" && $department ne "Maint") {
		&HTMLdie("Unknown department $department","So sorry Kimosabe..");
	}

	if ($action eq "display") {
		&Display();
	}
	elsif ($action eq "display_record") {
		&DisplayRecord();
	}
	elsif ($action eq "add") {
		&AddRecord();
	}
	elsif ($action eq "edit_record") {
		&EditRecord();
	}
	elsif ($action eq "save_edited_record") {
		&SaveEditedRecord();
	}
	elsif ($action eq "delete_record") {
		&DeleteRecord();
	}
	elsif ($action eq "find") {
		&FindRecord();
	}
	elsif ($action eq "mailit") {
		&Mailit();
	}
	else {
		&HTMLdie("Unknown action $action","So sorry Kimosabe..");
	}
	return;
}

#-------------------------------------------------------------------#
# Function Display
# Description: display last 5 days of journal entries

sub Display {
	# connect to database
	my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host","$user","$password",
			{'RaiseError' => 1}) || &HTMLdie("Unable to connect to database.");
	# prepare query
	my $sth = $dbh->prepare("SELECT * FROM journal 
		WHERE condition = \"Active\" && type = \"$department\" 
		ORDER BY id DESC");
	# print the page header
	&DisplayHeader("$airline_name $department_name Journal");
	&DisplayTable();
	# retrieve data
	$sth->execute();
	my ($data,$id,$date_time,$created_by,$journal);
	while ($data = $sth->fetchrow_arrayref) {
		$id=$$data[0];
		$date_time=$$data[1];
		$created_by=$$data[2];
		if (! $created_by) {
			$created_by = "&nbsp;";
		}
		$journal=$$data[3];

		my($pretty_date) = &FormatDate($date_time);

		print "<tr>";
		print "<td><a href=\"$prog_name?action=display_record&record=$id&department=$department\"> $id </a><br>";
		print "<a href=\"$prog_name?action=edit_record&record=$id&department=$department\">Edit </a><br>";
		print "<a href=\"$prog_name?action=delete_record&record=$id&department=$department\">Delete</a><br>";
		print "<a href=\"$prog_name?action=mailit&record=$id&department=$department\">Email</a></td>";
		print "<td> $pretty_date </td>";
		print "<td> $created_by </td>";
		print "<td> $journal </td>";
		print "</tr>";
	}
	# disconnect
	$sth->finish();
	$dbh->disconnect();
	# print the page footer
	print "</tbody>";
	print "</table>";
	&DisplayFooter();
}

#-------------------------------------------------------------------#
# Function DisplayRecord
# Description: display the day's journal

sub DisplayRecord {
	my $record = $input{'record'};
	my $department = $input{'department'};

	# connect to database
	my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host","$user","$password",
			{'RaiseError' => 1}) || &HTMLdie("Unable to connect to database.");
	# prepare query
	my $sth1 = $dbh->prepare("SELECT * from journal WHERE id = $record");

	# get the current record from the database and get it's date
	$sth1->execute();
	my @data = $sth1->fetchrow_array;
	my $journal_date_time = $data[1];
	my $journal_date = substr($journal_date_time,0,10);
	my $journal_hour = substr($journal_date_time,11,2);
	$sth1->finish();

	# check to see if we are past midnight and roll back the date if we are
	# note - this needs to be adjusted if the day start time is altered
	if ($journal_hour eq "00" || $journal_hour eq "01" || $journal_hour eq "02" || $journal_hour eq "03") {
		my $new_date = Date::EzDate->new($journal_date);
		$new_date--;
		$journal_date = $new_date->{'{year}-{month number base 1}-{day of month}'};
	}

	# now we figure the range from the retrieved date at $day_start through
	# the following day at $day_stop hours
	# create an EzDate object
	my $temp_date = Date::EzDate->new($journal_date);
	# increment 1 day
	$temp_date++;
	# put it in the proper format
	my $next_journal_date = $temp_date->{'{year}-{month number base 1}-{day of month}'};
	# finally set the range
	my $journal_start = "$journal_date $day_start";
	my $journal_stop = "$next_journal_date $day_stop";

	# prepare query and get the day's records
	my $sth2 = $dbh->prepare("SELECT * from journal WHERE 
			date_time >= \"$journal_start\" && date_time <= \"$journal_stop\" 
			&& type = \"$department\" 
			ORDER by date_time");
	$sth2->execute();

	# pretty things up
	my($pretty_journal_date) = &FormatDate2($journal_date);
	my($pretty_journal_start) = &FormatDate($journal_start);
	my($pretty_journal_stop) = &FormatDate($journal_stop);

	# print the page header
	&DisplayPlainHeader("$airline_name $department_name Journal for $pretty_journal_date");
	print "<table width = \"100%\">";
	print "<tr><td colspan=\"2\"><div align=\"center\">From $pretty_journal_start to $pretty_journal_stop</div></td></tr>";
	my $entry;
	while ($entry = $sth2->fetchrow_arrayref) {
		my $time = substr($$entry[1],11,8);
		print "<tr>";
		print "<td valign=\"top\" width = \"10%\">$time</td>";
		print "<td>$$entry[3]</td>";
		print "</tr>";
		print "<tr><td colspan=\"2\">&nbsp;</td></tr>";
	}

	# disconnect
	$sth2->finish();
	$dbh->disconnect();

	# print the page footer
	print "</table>";
	&DisplayFooter();
}

#-------------------------------------------------------------------#
# Function DeleteRecord
# Description: mark a record as deleted

sub DeleteRecord {
	my $department = $input{'department'};
	my $record = $input{'record'};

	# connect to database
	my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host","$user","$password",
			{'RaiseError' => 1}) || &HTMLdie("Unable to connect to database.");
	# prepare query
	my $sth = $dbh->prepare("DELETE FROM journal 
		WHERE id = \"$record\"
		");

	$sth->execute();
	$sth->finish();
	$dbh->disconnect();

	&Display();
}

#-------------------------------------------------------------------#
# Function FindRecord
# Description: find and display records

sub FindRecord {

	my $sdate_time = $input{'date_time'};
	my $screated_by = $input{'created_by'};
	my $sjournal = $input{'journal'};
	my $department = $input{'department'};
	if(!$sdate_time){$sdate_time="none"};
	if(!$screated_by){$screated_by="none"};
	if(!$sjournal){$sjournal="none"};

	# connect to database
	my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host","$user","$password",
			{'RaiseError' => 1}) || &HTMLdie("Unable to connect to database.");
	# prepare query
	my $sth = $dbh->prepare("SELECT * from journal
			WHERE (date_time LIKE \"%$sdate_time%\" &&	type = \"$department\" && condition = \"Active\") || 
		(created_by LIKE \"%$screated_by%\" &&	type = \"$department\" && condition = \"Active\") || 
		(journal LIKE \"%$sjournal%\" &&	type = \"$department\" && condition = \"Active\") 
		");

	# print the page header
	&DisplayHeader("$airline_name $department_name Journal");
	&DisplayTable();
	print "<tr>";
	print "<td>Searched for:</td>";
	print "<td>$sdate_time</td>";
	print "<td>$screated_by</td>";
	print "<td>$sjournal</td>";
	print "</tr>";

	# retrieve data
	$sth->execute();
	my($data,$id,$date_time,$created_by,$journal);
	while ($data = $sth->fetchrow_arrayref) {
		$id=$$data[0];
		$date_time=$$data[1];
		$created_by=$$data[2];
		$journal=$$data[3];

		print "<tr>";
		print "<td><a href=\"$prog_name?action=display_record&record=$id&department=$department\"> $id </a><br>";
		print "<a href=\"$prog_name?action=edit_record&record=$id&department=$department\">Edit </a><br>";
		print "<a href=\"$prog_name?action=delete_record&record=$id&department=$department\">Delete</a><br>";
		print "<a href=\"$prog_name?action=mailit&record=$id&department=$department\">Email</a></td>";
		print "<td> $date_time </td>";
		print "<td> $created_by </td>";
		print "<td> $journal </td>";
		print "</tr>";
	}

	# disconnect
	$sth->finish();
	$dbh->disconnect();

	# print the page footer
	print "</tbody>";
	print "</table>";
	&DisplayFooter();
}

#-------------------------------------------------------------------#
# Function EditRecord
# Description: Form for adding or editing a record

sub EditRecord {
	my $record = $input{'record'};

	# connect to database
	my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host","$user","$password",
			{'RaiseError' => 1}) || &HTMLdie("Unable to connect to database.");
	# prepare query
	my $sth = $dbh->prepare("SELECT * from journal WHERE id = $record");

	$sth->execute();
	my @data = $sth->fetchrow_array;

	$sth->finish();
	$dbh->disconnect();

	# replace <BR> with newlines in journal
	$data[3] =~ s/<br>/\n/g;
	$data[3] =~ s/<BR>/\n/g;

	my($pretty_timestamp) = &FormatDate($data[1]);


	&DisplayPlainHeader("$airline_name $department_name Journal Edit Record $record");
	# print record
	print "<form action=\"$prog_name\" method = \"post\">";
	print "<input type=\"hidden\" name=\"action\" value=\"save_edited_record\">";
	print "<input type=\"hidden\" name=\"record\" value=\"$record\">";
	print "<input type=\"hidden\" name=\"date_time\" value=\"$data[1]\">";
	print "<input type=\"hidden\" name=\"condition\" value=\"$data[4]\">";
	print "<input type=\"hidden\" name=\"type\" value=\"$data[5]\">";
	print "<input type=\"hidden\" name=\"department\" value=\"$department\">";
	print "<table width=\"100%\">";
	print "<tbody>";
	print "<tr>";
	print "<td align=\"center\"> <b>Condition:</b> $data[4] <b>Department:</b> $data[5] <b>Timestamp:</b> $pretty_timestamp</td>";
	print "</tr>";
	print "<tr>";
	print "<td align=\"center\"> BY:&nbsp;<input type=\"text\" name=\"created_by\" value = \"$data[2]\"> </td>";
	print "</tr>";
	print "<td align=\"center\"> <textarea name=\"journal\" cols=\"80\" rows=\"5\">$data[3]</textarea> </td>";
	print "</tr>";
	print "<td align=\"center\"> <input type=\"submit\" value=\"Save\"> </td>";
	print "</tr>";
	print "</tbody>";
	print "</table>";
	print "</form>";
	&DisplayFooter();

}

#-------------------------------------------------------------------#
# Function SaveEditedRecord
# Description: save record after editing

sub SaveEditedRecord {
	my $record = $input{'record'};
	my $created_by = $input{'created_by'};
	my $journal = $input{'journal'};

	# replace newlines with html tags in journal
	$journal =~ s/\n/<br>/g;

	# connect to database
	my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host","$user","$password",
			{'RaiseError' => 1}) || &HTMLdie("Unable to connect to database.");
	# prepare query
	my $sth = $dbh->prepare("UPDATE journal 
		SET created_by = \"$created_by\", journal = \"$journal\" 
		WHERE id = $record");

	$sth->execute();
	$sth->finish();
	$dbh->disconnect();

	&Display();

}

#-------------------------------------------------------------------#
# Function SaveEditedRecord
# Description: save record after editing

sub AddRecord {
	my $department = $input{'department'};
	my $created_by = $input{'created_by'};
	my $journal = $input{'journal'};

	my($sysdate,$systime,$sysyear,$sysmon,$sysmday) = &SysDate;
	my $date_time = "$sysdate $systime";

	# replace newlines with html tags in journal
	$journal =~ s/\n/<br>/g;

	# connect to database
	my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host","$user","$password",
			{'RaiseError' => 1}) || &HTMLdie("Unable to connect to database.");
	# prepare query
	my $sth = $dbh->prepare("INSERT INTO journal 
		SET created_by = \"$created_by\", journal = \"$journal\", 
		date_time = \"$date_time\", type = \"$department\"
		");

	$sth->execute();
	$sth->finish();
	$dbh->disconnect();

	&Display();

}

#-------------------------------------------------------------------#
# Function Mailit
# Description: email the current day's journal

sub Mailit {
	my $record = $input{'record'};
	my $department = $input{'department'};

	# get email addresses
	# connect to database
	my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host","$user","$password",
			{'RaiseError' => 1}) || &HTMLdie("Unable to connect to database.");
	# prepare query
	my $sth0 = $dbh->prepare("SELECT address from email 
		WHERE dept = \"$department\" && type = \"Active\"
		");
	$sth0->execute();
	my $addresses = "";
	while (my @address_data = $sth0->fetchrow_array) {
		$addresses .= $address_data[0];
		$addresses .= ",";
	}
	$sth0->finish();

	# now get the journal
	# prepare query
	my $sth1 = $dbh->prepare("SELECT * from journal WHERE id = $record");

	# get the current record from the database and get it's date
	$sth1->execute();
	my @data = $sth1->fetchrow_array;
	my $journal_date_time = $data[1];
	my $journal_date = substr($journal_date_time,0,10);
	my $journal_hour = substr($journal_date_time,11,2);
	$sth1->finish();

	# check to see if we are past midnight and roll back the date if we are
	# note - this needs to be adjusted if the day start time is altered
	if ($journal_hour eq "00" || $journal_hour eq "01" || $journal_hour eq "02" || $journal_hour eq "03") {
		my $new_date = Date::EzDate->new($journal_date);
		$new_date--;
		$journal_date = $new_date->{'{year}-{month number base 1}-{day of month}'};
	}

	# now we figure the range from the retrieved date at $day_start through
	# the following day at $day_stop hours
	# create an EzDate object
	my $temp_date = Date::EzDate->new($journal_date);
	# increment 1 day
	$temp_date++;
	# put it in the proper format
	my $next_journal_date = $temp_date->{'{year}-{month number base 1}-{day of month}'};
	# finally set the range
	my $journal_start = "$journal_date $day_start";
	my $journal_stop = "$next_journal_date $day_stop";

	# prepare query and get the day's records
	my $sth2 = $dbh->prepare("SELECT * from journal WHERE 
			date_time >= \"$journal_start\" && date_time <= \"$journal_stop\" 
			&& type = \"$department\" 
			ORDER by date_time");
	$sth2->execute();

	# pretty things up
	my($pretty_journal_date) = &FormatDate2($journal_date);
	my($pretty_journal_start) = &FormatDate($journal_start);
	my($pretty_journal_stop) = &FormatDate($journal_stop);

	# create the message body
	# 4/12/04 John Briggs requested we remove the plain text part of the message

	my $html_message = "<table width = \"100%\">";
	$html_message .= "<tr><td colspan=\"2\"><div align=\"center\">From $pretty_journal_start to $pretty_journal_stop</div></td></tr>";
	my $message = "$department_name Log attached.";
	#my $message = "From $journal_start to $journal_stop\n\n";
	my $entry;
	while ($entry = $sth2->fetchrow_arrayref) {
		my $time = substr($$entry[1],11,8);
		$html_message .= "<tr>";
		$html_message .= "<td valign=\"top\" width = \"10%\">$time</td>";
		$html_message .= "<td>$$entry[3]</td>";
		$html_message .= "</tr>";
		$html_message .= "<tr><td colspan=\"2\">&nbsp;</td></tr>";
		$$entry[3] =~ s/<br>/\n/g;
		$$entry[3] =~ s/<BR>/\n/g;
		#$message .= "$time\t$$entry[3]\n\n";
	}
	$html_message .= "</table>";

	$sth2->finish();
	$dbh->disconnect();

	# send the mail
	my $mail = new Mail::SendEasy(
			smtp => $mailserver ,
			user => $mailuser ,
			pass => $mailpass ,
			timeout => '120' ,
		);

	my $mail_status = $mail->send(
			from => $return_address ,
			to => $addresses ,
			subject => "$airline_name $department_name Log for $pretty_journal_date" ,
			msg => $message ,
			html => $html_message ,
		);

	if (!$mail_status) {&HTMLdie($mail->error);}

	&Display();
}

#-------------------------------------------------------------------#
# Function DisplayHeader
# Description: display page header

sub DisplayHeader {
	my($title)= @_ ;
	$title || ($title= "$airline_name") ;
	print "Content-type: text/html\n\n";
	print "<html>";
	print "<head>";
	print "<title>$title</title>";
	print "</head>";
	print "<body>";
	print "<table>";
	print "<tbody>";
	print "<tr>";
	print "<td> <img src=\"$logo_graphic\"> </td>";
	print "<td> <img src=\"$spacer_graphic\" width=\"50\"> </td>";
	print "<td> <div align=\"center\"><h1><font color=\"#022A96\">$airline_name</font></h1>";
	print "<font color=\"#979797\">$airline_address_1<br>$airline_address_2</font>";
	print "</div> </td>";
	print "</tr>";
	print "</tbody>";
	print "</table>";  
	print "<div align=\"center\">";
	print "<h2>$title</h2>";
	# input data form
	print "<form name=\"add_record\" action=$prog_name method=\"post\">";
	print "<input type=\"hidden\" name=\"action\" value=\"add\">";
	print "<input type=\"hidden\" name=\"department\" value=\"$department\">";
	print "<table width=\"100%\">";
	print "<tbody>";
	print "<tr>";
	print "<td align=\"center\"> BY:&nbsp;<input type=\"text\" name=\"created_by\" value=\"$input{'created_by'}\"> </td>";
	print "</tr>";
	print "<tr>";
	print "<td align=\"center\"> <textarea name=\"journal\" cols=\"80\" rows=\"5\"></textarea> </td>";
	print "</tr>";
	print "<tr>";
	print "<td align=\"center\"> <input type=\"submit\" value=\"Post to $department_name\">";
	print "&nbsp;<input type=button value=\"Manage Email List\" onClick=\"emailWindow=window.open(\'$manage_email_prog?department=$department\', \'emailWindow\', \'scrollbars=yes,menubar=yes,width=750,height=500,screenX=0,screenY=15\')\"></td>";
	print "</td>";
	print "</tr>";
	print "</tbody>";
	print "</table>";
	print "</form>";
	# spell check form
	print "<form name=\"spell\">";
	print "<script>";
	print "function sendtext()";
	print "{";
	print "var semi = new RegExp(\"\;\",\"g\");";
	print "document.send.checkme.value = (document.add_record.journal.value.replace(semi,\"\;\"));";
	print "document.send.form.value = \"add_record\";";
	print "document.send.field.value = \"journal\";";
	print "window.open('','SpellChecker',";
	print "'width=480,height=360,top=150,left=150,scrollbars=1,location=true');";
	print "document.send.submit();";
	print "}";
	print "</script>";
	print "<input type=\"button\" name=\"SpellCheck\" value=\"Spell Check\" onclick=\"sendtext()\">";
	print "</form>";

	print "<form name=\"send\" action=\"$spell_prog_name\" method=\"post\" target=\"SpellChecker\">";
	print "<input type=\"hidden\" name=\"checkme\">";
	print "<input type=\"hidden\" name=\"form\">";
	print "<input type=\"hidden\" name=\"field\">";
	print "<input type=\"hidden\" name=\"spell\" value=\"check\">";
	print "</form>";
	print "<a href=\"$prog_name?department=$department\">Refresh Page</a>";
	print "<p></p>";
	return;
}

#-------------------------------------------------------------------#
# Function DisplayPlainHeader
# Description: display header without form

sub DisplayPlainHeader {
	my($title)= @_ ;
	$title || ($title= "$airline_name") ;
	print "Content-type: text/html\n\n";
	print "<html>";
	print "<head>";
	print "<title>$title</title>";
	print "</head>";
	print "<body>";
	print "<div align=\"center\">";
	print "<h2>$title</h2>";
}

#-------------------------------------------------------------------#
# Function DisplayTable
# Description: display beginning of reporting table

sub DisplayTable {
	print "<table border=\"1\" width=\"80%\">";
	print "<tbody>";
	print "<tr>";
	print "<td> Edit<br>Delete </td>";
	print "<td> Date<br>Time </td>";
	print "<td> By </td>";
	print "<td> Journal </td>";
	print "</tr>";
	print "<tr> <form action=\"$prog_name\" method=\"post\">";
	print "<input type=\"hidden\" name=\"action\" value=\"find\">";
	print "<input type=\"hidden\" name=\"department\" value=\"$department\">";
	print "<td width=\"10%\"> <input type=\"submit\" name=\"submit\" value=\"Find\"> </td>";
	print "<td width=\"10%\"> <input type=\"text\" name=\"date_time\" size=\"12\"> </td>";
	print "<td width=\"10%\"> <input type=\"text\" name=\"created_by\" size=\"12\"> </td>";
	print "<td width=\"70%\"> <input type=\"text\" name=\"journal\" size=\"40\"> </td>";
	print "</tr> </form>";
	return;
}

#-------------------------------------------------------------------#
# Function DisplayFooter
# Description: display page footer

sub DisplayFooter {
	print "<table width=\"100%\">";
	print "<tbody>";
	print "<tr>";
	print "<td><div align=\"center\"><font size=\"-3\"> $copyright </font></div></td>";
	print "</tbody>";
	print "</table>";
	print "</div>";
	print "</body>";
	print "</html>";
	return;
}

#-------------------------------------------------------------------#
# Function FormatDate
# Description: format dates for display
#
sub FormatDate {
	# usage: my $pretty_date = &FormatDate("YYYY-MM-DD HH:MM:SS");

	my($date_time_in) = @_;
	my $temp_date = Date::EzDate->new($date_time_in);
	my $formatted_date = $temp_date->{'full'};

	return($formatted_date);

}

#-------------------------------------------------------------------#
# Function FormatDate
# Description: format dates for display
#
sub FormatDate2 {
	# usage: my $pretty_date = &FormatDate("YYYY-MM-DD HH:MM:SS");

	my($date_time_in) = @_;
	my $temp_date = Date::EzDate->new($date_time_in);
	my $formatted_date = $temp_date->{'{weekday short} {month short} {day of month}, {year}'};

	return($formatted_date);

}

#-------------------------------------------------------------------#
# Function SysDate
# Description: get current date/time
#
sub SysDate {
	# usage:  my($sysdate,$systime,$sysyear,$sysmon,$sysmday) = &SysDate;

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
	if (length ($min) == 1) {$min = '0'.$min;}
	if (length ($sec) == 1) {$sec = '0'.$sec;}
	# since localtime returns the month as 0-11
	$mon = $mon + 1;
	if (length ($mon) == 1) {$mon = '0'.$mon;}
	if (length ($mday) == 1) {$mday = '0'.$mday;}
	# since localtime returns the year as the number of years since 1900
	# ie year is 100 in the year 2000 (so is y2k OK)
	$year = $year + 1900;
	my $date = "$year-$mon-$mday";
	my $time = "$hour:$min:$sec";
	return($date,$time,$year,$mon,$mday);
}

#-------------------------------------------------------------------#
# Function getcgivars
# Read all CGI vars into an associative array.
# courtesy James Marshall james@jmarshall.com http://www.jmarshall.com/easy/cgi/
# If multiple input fields have the same name, they are concatenated into
#   one array element and delimited with the \0 character (which fails if
	#   the input has any \0 characters, very unlikely but conceivably possible).
# Currently only supports Content-Type of application/x-www-form-urlencoded.
sub getcgivars {
	local($in, %in) ;
	local($name, $value) ;


		# First, read entire string of CGI vars into $in
	if ( ($ENV{'REQUEST_METHOD'} eq 'GET') ||
			($ENV{'REQUEST_METHOD'} eq 'HEAD') ) {
		$in= $ENV{'QUERY_STRING'} ;

	} elsif ($ENV{'REQUEST_METHOD'} eq 'POST') {
		if ($ENV{'CONTENT_TYPE'}=~ m#^application/x-www-form-urlencoded$#i) {
			length($ENV{'CONTENT_LENGTH'})
				|| &HTMLdie("No Content-Length sent with the POST request.") ;
			read(STDIN, $in, $ENV{'CONTENT_LENGTH'}) ;

		} else { 
			&HTMLdie("Unsupported Content-Type: $ENV{'CONTENT_TYPE'}") ;
		}

	} else {
		&HTMLdie("Script was called with unsupported REQUEST_METHOD.") ;
	}

		# Resolve and unencode name/value pairs into %in
	foreach (split('&', $in)) {
		s/\+/ /g ;
		($name, $value)= split('=', $_, 2) ;
		$name=~ s/%(..)/chr(hex($1))/ge ;
		$value=~ s/%(..)/chr(hex($1))/ge ;
		$in{$name}.= "\0" if defined($in{$name}) ;  # concatenate multiple vars
		$in{$name}.= $value ;
	}

	return %in ;

}

#-------------------------------------------------------------------#
# Function HTMLdie
# Description: Die, outputting HTML error page
# If no $title, use a default title
sub HTMLdie {
	local($msg,$title)= @_ ;
	$title || ($title= "CGI Error") ;
	print "Content-type: text/html\n\n";
	print "<html>";
	print "<head>";
	print "<title>$title</title>";
	print "</head>";
	print "<body>";
	print "<h1>$title</h1>";
	print "<h3>$msg</h3>";
	print "<form>";
	print "<input type=button name=\"BackButton\" value=\"<- Back\" id=\"Button1\" onClick=\"history.back()\">";
	print "</form>";
	print "</body>";
	print "</html>";

	exit ;
}
